@@ -1,39 +1,31 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
+    Copyright (C) 1997-2006 Sam Lantinga
 
     This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
+    modify it under the terms of the GNU Lesser General Public
     License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
+    version 2.1 of the License, or (at your option) any later version.
 
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
+    Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Library General Public
-    License along with this library; if not, write to the Free
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
     Sam Lantinga
-    slouken@devolution.com
+    slouken@libsdl.org
 */
-
-#ifdef SAVE_RCSID
-static char rcsid =
- "@(#) $Id: SDL_dx5audio.c,v 1.4 2001/10/25 08:45:17 slouken Exp $";
-#endif
+#include "SDL_config.h"
 
 /* Allow access to a raw mixing buffer */
 
-#include <stdio.h>
-
-#include "SDL_types.h"
-#include "SDL_error.h"
 #include "SDL_timer.h"
 #include "SDL_audio.h"
-#include "SDL_audio_c.h"
+#include "../SDL_audio_c.h"
 #include "SDL_dx5audio.h"
 
 /* Define this if you want to use DirectX 6 DirectSoundNotify interface */
@@ -63,7 +55,7 @@ static int Audio_Available(void)
 
 	/* Version check DSOUND.DLL (Is DirectX okay?) */
 	dsound_ok = 0;
-	DSoundDLL = LoadLibrary("DSOUND.DLL");
+	DSoundDLL = LoadLibrary(TEXT("DSOUND.DLL"));
 	if ( DSoundDLL != NULL ) {
 		/* We just use basic DirectSound, we're okay */
 		/* Yay! */
@@ -95,7 +87,7 @@ static int Audio_Available(void)
 		 * to fall back to the DIB driver. */
 		if (dsound_ok) {
 			/* DirectSoundCaptureCreate was added in DX5 */
-			if (!GetProcAddress(DSoundDLL, "DirectSoundCaptureCreate"))
+			if (!GetProcAddress(DSoundDLL, TEXT("DirectSoundCaptureCreate")))
 				dsound_ok = 0;
 
 		}
@@ -121,10 +113,10 @@ static int DX5_Load(void)
 	int status;
 
 	DX5_Unload();
-	DSoundDLL = LoadLibrary("DSOUND.DLL");
+	DSoundDLL = LoadLibrary(TEXT("DSOUND.DLL"));
 	if ( DSoundDLL != NULL ) {
 		DSoundCreate = (void *)GetProcAddress(DSoundDLL,
-					"DirectSoundCreate");
+					TEXT("DirectSoundCreate"));
 	}
 	if ( DSoundDLL && DSoundCreate ) {
 		status = 0;
@@ -138,8 +130,8 @@ static int DX5_Load(void)
 static void Audio_DeleteDevice(SDL_AudioDevice *device)
 {
 	DX5_Unload();
-	free(device->hidden);
-	free(device);
+	SDL_free(device->hidden);
+	SDL_free(device);
 }
 
 static SDL_AudioDevice *Audio_CreateDevice(int devindex)
@@ -152,20 +144,20 @@ static SDL_AudioDevice *Audio_CreateDevice(int devindex)
 	}
 
 	/* Initialize all variables that we clean on shutdown */
-	this = (SDL_AudioDevice *)malloc(sizeof(SDL_AudioDevice));
+	this = (SDL_AudioDevice *)SDL_malloc(sizeof(SDL_AudioDevice));
 	if ( this ) {
-		memset(this, 0, (sizeof *this));
+		SDL_memset(this, 0, (sizeof *this));
 		this->hidden = (struct SDL_PrivateAudioData *)
-				malloc((sizeof *this->hidden));
+				SDL_malloc((sizeof *this->hidden));
 	}
 	if ( (this == NULL) || (this->hidden == NULL) ) {
 		SDL_OutOfMemory();
 		if ( this ) {
-			free(this);
+			SDL_free(this);
 		}
 		return(0);
 	}
-	memset(this->hidden, 0, (sizeof *this->hidden));
+	SDL_memset(this->hidden, 0, (sizeof *this->hidden));
 
 	/* Set the function pointers */
 	this->OpenAudio = DX5_OpenAudio;
@@ -189,7 +181,7 @@ AudioBootStrap DSOUND_bootstrap = {
 static void SetDSerror(const char *function, int code)
 {
 	static const char *error;
-	static char  errbuf[BUFSIZ];
+	static char  errbuf[1024];
 
 	errbuf[0] = 0;
 	switch (code) {
@@ -228,12 +220,13 @@ static void SetDSerror(const char *function, int code)
 			error = "Function not supported";
 			break;
 		default:
-			sprintf(errbuf, "%s: Unknown DirectSound error: 0x%x",
+			SDL_snprintf(errbuf, SDL_arraysize(errbuf),
+			         "%s: Unknown DirectSound error: 0x%x",
 								function, code);
 			break;
 	}
 	if ( ! errbuf[0] ) {
-		sprintf(errbuf, "%s: %s", function, error);
+		SDL_snprintf(errbuf, SDL_arraysize(errbuf), "%s: %s", function, error);
 	}
 	SDL_SetError("%s", errbuf);
 	return;
@@ -390,7 +383,7 @@ static void DX5_WaitDone(_THIS)
 	/* Wait for the playing chunk to finish */
 	stream = this->GetAudioBuf(this);
 	if ( stream != NULL ) {
-		memset(stream, silence, mixlen);
+		SDL_memset(stream, silence, mixlen);
 		this->PlayAudio(this);
 	}
 	this->WaitAudio(this);
@@ -416,6 +409,7 @@ static void DX5_CloseAudio(_THIS)
 	}
 }
 
+#ifdef USE_PRIMARY_BUFFER
 /* This function tries to create a primary audio buffer, and returns the
    number of audio chunks available in the created buffer.
 */
@@ -438,7 +432,7 @@ static int CreatePrimary(LPDIRECTSOUND sndObj, HWND focus,
 	}
 
 	/* Try to create the primary buffer */
-	memset(&format, 0, sizeof(format));
+	SDL_memset(&format, 0, sizeof(format));
 	format.dwSize = sizeof(format);
 	format.dwFlags=(DSBCAPS_PRIMARYBUFFER|DSBCAPS_GETCURRENTPOSITION2);
 	format.dwFlags |= DSBCAPS_STICKYFOCUS;
@@ -454,7 +448,7 @@ static int CreatePrimary(LPDIRECTSOUND sndObj, HWND focus,
 	}
 
 	/* Check the size of the fragment buffer */
-	memset(&caps, 0, sizeof(caps));
+	SDL_memset(&caps, 0, sizeof(caps));
 	caps.dwSize = sizeof(caps);
 	result = IDirectSoundBuffer_GetCaps(*sndbuf, &caps);
 	if ( result != DS_OK ) {
@@ -489,6 +483,7 @@ static int CreatePrimary(LPDIRECTSOUND sndObj, HWND focus,
 	}
 	return(numchunks);
 }
+#endif /* USE_PRIMARY_BUFFER */
 
 /* This function tries to create a secondary audio buffer, and returns the
    number of audio chunks available in the created buffer.
@@ -518,7 +513,7 @@ static int CreateSecondary(LPDIRECTSOUND sndObj, HWND focus,
 	}
 
 	/* Try to create the secondary buffer */
-	memset(&format, 0, sizeof(format));
+	SDL_memset(&format, 0, sizeof(format));
 	format.dwSize = sizeof(format);
 	format.dwFlags = DSBCAPS_GETCURRENTPOSITION2;
 #ifdef USE_POSITION_NOTIFY
@@ -552,9 +547,9 @@ static int CreateSecondary(LPDIRECTSOUND sndObj, HWND focus,
 	                                 DSBLOCK_ENTIREBUFFER);
 	if ( result == DS_OK ) {
 		if ( wavefmt->wBitsPerSample == 8 ) {
-			memset(pvAudioPtr1, 0x80, dwAudioBytes1);
+			SDL_memset(pvAudioPtr1, 0x80, dwAudioBytes1);
 		} else {
-			memset(pvAudioPtr1, 0x00, dwAudioBytes1);
+			SDL_memset(pvAudioPtr1, 0x00, dwAudioBytes1);
 		}
 		IDirectSoundBuffer_Unlock(*sndbuf,
 		                          (LPVOID)pvAudioPtr1, dwAudioBytes1,
@@ -586,7 +581,7 @@ static int CreateAudioEvent(_THIS)
 	}
 
 	/* Allocate the notify structures */
-	notify_positions = (DSBPOSITIONNOTIFY *)malloc(NUM_BUFFERS*
+	notify_positions = (DSBPOSITIONNOTIFY *)SDL_malloc(NUM_BUFFERS*
 					sizeof(*notify_positions));
 	if ( notify_positions == NULL ) {
 		goto done;
@@ -622,7 +617,7 @@ static int DX5_OpenAudio(_THIS, SDL_AudioSpec *spec)
 	WAVEFORMATEX waveformat;
 
 	/* Set basic WAVE format parameters */
-	memset(&waveformat, 0, sizeof(waveformat));
+	SDL_memset(&waveformat, 0, sizeof(waveformat));
 	waveformat.wFormatTag = WAVE_FORMAT_PCM;
 
 	/* Determine the audio parameters from the AudioSpec */
@@ -662,10 +657,12 @@ static int DX5_OpenAudio(_THIS, SDL_AudioSpec *spec)
 
 	/* Create the audio buffer to which we write */
 	NUM_BUFFERS = -1;
+#ifdef USE_PRIMARY_BUFFER
 	if ( mainwin ) {
 		NUM_BUFFERS = CreatePrimary(sound, mainwin, &mixbuf,
 						&waveformat, spec->size);
 	}
+#endif /* USE_PRIMARY_BUFFER */
 	if ( NUM_BUFFERS < 0 ) {
 		NUM_BUFFERS = CreateSecondary(sound, mainwin, &mixbuf,
 						&waveformat, spec->size);
