@@ -1,44 +1,36 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
+    Copyright (C) 1997-2006 Sam Lantinga
 
     This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
+    modify it under the terms of the GNU Lesser General Public
     License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
+    version 2.1 of the License, or (at your option) any later version.
 
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
+    Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Library General Public
-    License along with this library; if not, write to the Free
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
     Sam Lantinga
-    slouken@devolution.com
+    slouken@libsdl.org
 */
-
-#ifdef SAVE_RCSID
-static char rcsid =
- "@(#) $Id: SDL_cgxvideo.h,v 1.3 2001/05/10 20:13:29 hercules Exp $";
-#endif
+#include "SDL_config.h"
 
 #ifndef _SDL_cgxvideo_h
 #define _SDL_cgxvideo_h
 
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-
 #include <exec/exec.h>
 #include <cybergraphx/cybergraphics.h>
 #include <graphics/scale.h>
 #include <graphics/gfx.h>
 #include <intuition/intuition.h>
-#ifdef __SASC
+#if defined(__SASC) || defined(STORMC4_WOS)
 #include <proto/exec.h>
 #include <proto/cybergraphics.h>
 #include <proto/graphics.h>
@@ -52,9 +44,9 @@ static char rcsid =
 #include <inline/console.h>
 #endif
 
-#include "mydebug.h"
 #include "SDL_mouse.h"
-#include "SDL_sysvideo.h"
+#include "../SDL_sysvideo.h"
+#include "mydebug.h"
 
 #define USE_CGX_WRITELUTPIXEL
 
@@ -75,10 +67,6 @@ struct SDL_PrivateVideoData {
     Uint8 *Ximage;		/* The X image for our window */
     int swap_pixels;		/* Flag: true if display is swapped endian */
 
-    /* The current width and height of the fullscreen mode */
-    int current_w;
-    int current_h;
-
     /* Support for internal mouse warping */
     struct {
         int x;
@@ -140,8 +128,6 @@ struct SDL_PrivateVideoData {
 #define SDL_Ximage		(this->hidden->Ximage)
 #define SDL_GC			(this->hidden->gc)
 #define swap_pixels		(this->hidden->swap_pixels)
-#define current_w		(this->hidden->current_w)
-#define current_h		(this->hidden->current_h)
 #define mouse_last		(this->hidden->mouse_last)
 #define mouse_accel		(this->hidden->mouse_accel)
 #define mouse_relative		(this->hidden->mouse_relative)
@@ -177,6 +163,7 @@ struct private_hwdata
 	APTR lock;
 	struct SDL_VideoDevice *videodata;
 	APTR mask;
+	int allocated;
 };
 
 int CGX_CheckHWBlit(_THIS,SDL_Surface *src,SDL_Surface *dst);
