@@ -1,42 +1,35 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997-2004 Sam Lantinga
+    Copyright (C) 1997-2006 Sam Lantinga
 
     This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
+    modify it under the terms of the GNU Lesser General Public
     License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
+    version 2.1 of the License, or (at your option) any later version.
 
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
+    Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Library General Public
-    License along with this library; if not, write to the Free
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
     Sam Lantinga
     slouken@libsdl.org
 */
-
-#ifdef SAVE_RCSID
-static char rcsid =
- "@(#) $Id: SDL_wingl.c,v 1.14 2004/01/04 16:49:26 slouken Exp $";
-#endif
-
-#include <string.h>
+#include "SDL_config.h"
 
 /* WGL implementation of SDL OpenGL support */
 
-#ifdef HAVE_OPENGL
+#if SDL_VIDEO_OPENGL
 #include "SDL_opengl.h"
 #endif
-#include "SDL_error.h"
 #include "SDL_lowvideo.h"
 #include "SDL_wingl_c.h"
 
-#ifdef HAVE_OPENGL
+#if SDL_VIDEO_OPENGL
 #define DEFAULT_GL_DRIVER_PATH "OPENGL32.DLL"
 #endif
 
@@ -44,26 +37,27 @@ static char rcsid =
 static int WIN_GL_ResetWindow(_THIS)
 {
 	int status = 0;
-	int can_reset = 1;
 
-	/* If we were passed a window, then we can't create a new one */
-	if ( SDL_windowid ) {
-		can_reset = 0;
-	}
 #ifndef _WIN32_WCE /* FIXME WinCE needs the UNICODE version of CreateWindow() */
-	if ( can_reset ) {
+	/* This doesn't work with DirectX code (see CVS comments) */
+	/* If we were passed a window, then we can't create a new one */
+	if ( !SDL_windowid && SDL_strcmp(this->name, "windib") == 0 ) {
 		/* Save the existing window attributes */
 		LONG style;
 		RECT rect = { 0, 0, 0, 0 };
 		style = GetWindowLong(SDL_Window, GWL_STYLE);
 		GetWindowRect(SDL_Window, &rect);
 		DestroyWindow(SDL_Window);
+		WIN_FlushMessageQueue();
+
 		SDL_Window = CreateWindow(SDL_Appname, SDL_Appname,
 		                          style,
-					  rect.left, rect.top,
-                                          (rect.right-rect.left)+1,
-                                          (rect.top-rect.bottom)+1,
+		                          rect.left, rect.top,
+		                          (rect.right-rect.left)+1,
+		                          (rect.top-rect.bottom)+1,
 		                          NULL, NULL, SDL_Instance, NULL);
+		WIN_FlushMessageQueue();
+
 		if ( SDL_Window ) {
 			this->SetCaption(this, this->wm_title, this->wm_icon);
 		} else {
@@ -79,7 +73,42 @@ static int WIN_GL_ResetWindow(_THIS)
 	return(status);
 }
 
-#ifdef HAVE_OPENGL
+#if SDL_VIDEO_OPENGL
+
+static int ExtensionSupported(const char *extension, const char *extensions)
+{
+	const char *start;
+	const char *where, *terminator;
+
+	/* Extension names should not have spaces. */
+	where = SDL_strchr(extension, ' ');
+	if ( where || *extension == '\0' )
+	      return 0;
+	
+	if ( ! extensions )
+		return 0;
+
+	/* It takes a bit of care to be fool-proof about parsing the
+	 *      OpenGL extensions string. Don't be fooled by sub-strings,
+	 *           etc. */
+	
+	start = extensions;
+	
+	for (;;)
+	{
+		where = SDL_strstr(start, extension);
+		if (!where) break;
+		
+		terminator = where + SDL_strlen(extension);
+		if (where == start || *(where - 1) == ' ')
+	        if (*terminator == ' ' || *terminator == '\0') return 1;
+
+		start = terminator;
+	}
+	
+	return 0;
+}
+
 static void Init_WGL_ARB_extensions(_THIS)
 {
 	HWND hwnd;
@@ -87,10 +116,13 @@ static void Init_WGL_ARB_extensions(_THIS)
 	HGLRC hglrc;
 	int pformat;
 	const char * (WINAPI *wglGetExtensionsStringARB)(HDC) = 0;
+	const char *extensions;
 	
 	hwnd = CreateWindow(SDL_Appname, SDL_Appname, WS_POPUP | WS_DISABLED,
 	                    0, 0, 10, 10,
-	                    NULL, NULL, SDL_Instance,NULL);
+	                    NULL, NULL, SDL_Instance, NULL);
+	WIN_FlushMessageQueue();
+
 	hdc = GetDC(hwnd);
 
 	pformat = ChoosePixelFormat(hdc, &GL_pfd);
@@ -104,7 +136,14 @@ static void Init_WGL_ARB_extensions(_THIS)
 	wglGetExtensionsStringARB = (const char * (WINAPI *)(HDC))
 		this->gl_data->wglGetProcAddress("wglGetExtensionsStringARB");
 
-	if(wglGetExtensionsStringARB && strstr(wglGetExtensionsStringARB(hdc),"WGL_ARB_pixel_format")) {
+	if( wglGetExtensionsStringARB ) {
+		extensions = wglGetExtensionsStringARB(hdc);
+	} else {
+		extensions = NULL;
+	}
+
+	this->gl_data->WGL_ARB_pixel_format = 0;
+	if( ExtensionSupported("WGL_ARB_pixel_format", extensions) ) {
 		this->gl_data->wglChoosePixelFormatARB =
 			(BOOL (WINAPI *)(HDC, const int *, const FLOAT *, UINT, int *, UINT *))
 			this->gl_data->wglGetProcAddress("wglChoosePixelFormatARB");
@@ -113,12 +152,9 @@ static void Init_WGL_ARB_extensions(_THIS)
 			this->gl_data->wglGetProcAddress("wglGetPixelFormatAttribivARB");
 
 		if( (this->gl_data->wglChoosePixelFormatARB != NULL) &&
-		    (this->gl_data->wglGetPixelFormatAttribivARB != NULL) )
-			this->gl_data->wgl_arb_pixel_format = 1;
-		else 
-			this->gl_data->wgl_arb_pixel_format = 0;
-	} else {
-		this->gl_data->wgl_arb_pixel_format = 0;
+		    (this->gl_data->wglGetPixelFormatAttribivARB != NULL) ) {
+			this->gl_data->WGL_ARB_pixel_format = 1;
+		}
 	}
 	
 	if ( hglrc ) {
@@ -127,18 +163,22 @@ static void Init_WGL_ARB_extensions(_THIS)
 	}
 	ReleaseDC(hwnd, hdc);
 	DestroyWindow(hwnd);
+	WIN_FlushMessageQueue();
 }
-#endif /* !HAVE_OPENGL */
+
+#endif /* SDL_VIDEO_OPENGL */
 
 int WIN_GL_SetupWindow(_THIS)
 {
 	int retval;
-#ifdef HAVE_OPENGL
+#if SDL_VIDEO_OPENGL
 	int i;
 	unsigned int matching;
 	int iAttribs[64];
 	int *iAttr;
 	float fAttribs[1] = { 0 };
+	const GLubyte *(WINAPI *glGetStringFunc)(GLenum);
+	const char *wglext;
 
 	/* load the gl driver from a default path */
 	if ( ! this->gl_config.driver_loaded ) {
@@ -157,7 +197,7 @@ int WIN_GL_SetupWindow(_THIS)
 		}
 
 		/* Set up the pixel format descriptor with our needed format */
-		memset(&GL_pfd, 0, sizeof(GL_pfd));
+		SDL_memset(&GL_pfd, 0, sizeof(GL_pfd));
 		GL_pfd.nSize = sizeof(GL_pfd);
 		GL_pfd.nVersion = 1;
 		GL_pfd.dwFlags = (PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL);
@@ -205,10 +245,8 @@ int WIN_GL_SetupWindow(_THIS)
 			*iAttr++ = this->gl_config.alpha_size;
 		}
 
-		if ( this->gl_config.double_buffer ) {
-			*iAttr ++ = WGL_DOUBLE_BUFFER_ARB;
-			*iAttr ++ = GL_TRUE;
-		}
+		*iAttr++ = WGL_DOUBLE_BUFFER_ARB;
+		*iAttr++ = this->gl_config.double_buffer;
 
 		*iAttr++ = WGL_DEPTH_BITS_ARB;
 		*iAttr++ = this->gl_config.depth_size;
@@ -240,7 +278,7 @@ int WIN_GL_SetupWindow(_THIS)
 
 		if ( this->gl_config.stereo ) {
 			*iAttr++ = WGL_STEREO_ARB;
-			*iAttr++ = this->gl_config.stereo;
+			*iAttr++ = GL_TRUE;
 		}
 
 		if ( this->gl_config.multisamplebuffers ) {
@@ -253,14 +291,19 @@ int WIN_GL_SetupWindow(_THIS)
 			*iAttr++ = this->gl_config.multisamplesamples;
 		}
 
+		if ( this->gl_config.accelerated >= 0 ) {
+			*iAttr++ = WGL_ACCELERATION_ARB;
+			*iAttr++ = (this->gl_config.accelerated ? WGL_GENERIC_ACCELERATION_ARB : WGL_NO_ACCELERATION_ARB);
+		}
+
 		*iAttr = 0;
 
 		/* Choose and set the closest available pixel format */
-		if ( !this->gl_data->wgl_arb_pixel_format ||
+		if ( !this->gl_data->WGL_ARB_pixel_format ||
 		     !this->gl_data->wglChoosePixelFormatARB(GL_hdc, iAttribs, fAttribs, 1, &pixel_format, &matching) ||
 		     !matching ) {
 			pixel_format = ChoosePixelFormat(GL_hdc, &GL_pfd);
-			this->gl_data->wgl_arb_pixel_format = 0;
+			this->gl_data->WGL_ARB_pixel_format = 0;
 		}
 		if ( !pixel_format ) {
 			SDL_SetError("No matching GL pixel format available");
@@ -287,7 +330,31 @@ int WIN_GL_SetupWindow(_THIS)
 		SDL_SetError("Unable to create GL context");
 		return(-1);
 	}
+	if ( WIN_GL_MakeCurrent(this) < 0 ) {
+		return(-1);
+	}
 	gl_active = 1;
+
+	/* Vsync control under Windows.  Checking glGetString here is
+	 * somewhat a documented and reliable hack - it was originally
+	 * as a feature added by mistake, but since so many people rely
+	 * on it, it will not be removed.  strstr should be safe here.*/
+	glGetStringFunc = WIN_GL_GetProcAddress(this, "glGetString");
+	if ( glGetStringFunc ) {
+		wglext = (const char *)glGetStringFunc(GL_EXTENSIONS);
+	} else {
+		/* Uh oh, something is seriously wrong here... */
+		wglext = NULL;
+	}
+	if ( !wglext || !SDL_strstr(wglext, "WGL_EXT_swap_control") ) {
+		this->gl_data->wglSwapIntervalEXT = NULL;
+		this->gl_data->wglGetSwapIntervalEXT = NULL;
+	}
+	if ( this->gl_config.swap_control >= 0 ) {
+		if ( this->gl_data->wglSwapIntervalEXT ) {
+			this->gl_data->wglSwapIntervalEXT(this->gl_config.swap_control);
+		}
+	}
 #else
 	SDL_SetError("WIN driver not configured with OpenGL");
 #endif
@@ -301,7 +368,7 @@ int WIN_GL_SetupWindow(_THIS)
 
 void WIN_GL_ShutDown(_THIS)
 {
-#ifdef HAVE_OPENGL
+#if SDL_VIDEO_OPENGL
 	/* Clean up OpenGL */
 	if ( GL_hrc ) {
 		this->gl_data->wglMakeCurrent(NULL, NULL);
@@ -315,10 +382,10 @@ void WIN_GL_ShutDown(_THIS)
 	gl_active = 0;
 
 	WIN_GL_UnloadLibrary(this);
-#endif /* HAVE_OPENGL */
+#endif /* SDL_VIDEO_OPENGL */
 }
 
-#ifdef HAVE_OPENGL
+#if SDL_VIDEO_OPENGL
 
 /* Make the current context active */
 int WIN_GL_MakeCurrent(_THIS)
@@ -338,7 +405,7 @@ int WIN_GL_GetAttribute(_THIS, SDL_GLattr attrib, int* value)
 {
 	int retval;
 	
-	if ( this->gl_data->wgl_arb_pixel_format ) {
+	if ( this->gl_data->WGL_ARB_pixel_format ) {
 		int wgl_attrib;
 
 		switch(attrib) {
@@ -387,6 +454,22 @@ int WIN_GL_GetAttribute(_THIS, SDL_GLattr attrib, int* value)
 		    case SDL_GL_MULTISAMPLESAMPLES:
 			wgl_attrib = WGL_SAMPLES_ARB;
 			break;
+		    case SDL_GL_ACCELERATED_VISUAL:
+			wgl_attrib = WGL_ACCELERATION_ARB;
+			this->gl_data->wglGetPixelFormatAttribivARB(GL_hdc, pixel_format, 0, 1, &wgl_attrib, value);
+			if ( *value == WGL_NO_ACCELERATION_ARB ) {
+				*value = SDL_FALSE;
+			} else {
+				*value = SDL_TRUE;
+			}
+			return 0;
+			break;
+		    case SDL_GL_SWAP_CONTROL:
+			if ( this->gl_data->wglGetSwapIntervalEXT ) {
+				return this->gl_data->wglGetSwapIntervalEXT();
+			} else {
+				return -1;
+			}
 		    default:
 			return(-1);
 		}
@@ -444,6 +527,12 @@ int WIN_GL_GetAttribute(_THIS, SDL_GLattr attrib, int* value)
 			*value = 0;
 		}
 		break;
+	    case SDL_GL_MULTISAMPLEBUFFERS:
+		*value = 0;
+		break;
+	    case SDL_GL_MULTISAMPLESAMPLES:
+		*value = 1;
+		break;
 	    default:
 		retval = -1;
 		break;
@@ -467,6 +556,8 @@ void WIN_GL_UnloadLibrary(_THIS)
 		this->gl_data->wglMakeCurrent = NULL;
 		this->gl_data->wglChoosePixelFormatARB = NULL;
 		this->gl_data->wglGetPixelFormatAttribivARB = NULL;
+		this->gl_data->wglSwapIntervalEXT = NULL;
+		this->gl_data->wglGetSwapIntervalEXT = NULL;
 
 		this->gl_config.dll_handle = NULL;
 		this->gl_config.driver_loaded = 0;
@@ -496,7 +587,7 @@ int WIN_GL_LoadLibrary(_THIS, const char* path)
 	WIN_GL_UnloadLibrary(this);
 
 	/* Load new function pointers */
-	memset(this->gl_data, 0, sizeof(*this->gl_data));
+	SDL_memset(this->gl_data, 0, sizeof(*this->gl_data));
 	this->gl_data->wglGetProcAddress = (void * (WINAPI *)(const char *))
 		GetProcAddress(handle, "wglGetProcAddress");
 	this->gl_data->wglCreateContext = (HGLRC (WINAPI *)(HDC))
@@ -505,6 +596,10 @@ int WIN_GL_LoadLibrary(_THIS, const char* path)
 		GetProcAddress(handle, "wglDeleteContext");
 	this->gl_data->wglMakeCurrent = (BOOL (WINAPI *)(HDC, HGLRC))
 		GetProcAddress(handle, "wglMakeCurrent");
+	this->gl_data->wglSwapIntervalEXT = (void (WINAPI *)(int))
+		GetProcAddress(handle, "wglSwapIntervalEXT");
+	this->gl_data->wglGetSwapIntervalEXT = (int (WINAPI *)(void))
+		GetProcAddress(handle, "wglGetSwapIntervalEXT");
 
 	if ( (this->gl_data->wglGetProcAddress == NULL) ||
 	     (this->gl_data->wglCreateContext == NULL) ||
@@ -516,7 +611,7 @@ int WIN_GL_LoadLibrary(_THIS, const char* path)
 	}
 
 	this->gl_config.dll_handle = handle;
-	strcpy(this->gl_config.driver_path, path);
+	SDL_strlcpy(this->gl_config.driver_path, path, SDL_arraysize(this->gl_config.driver_path));
 	this->gl_config.driver_loaded = 1;
 	return 0;
 }
@@ -534,4 +629,4 @@ void *WIN_GL_GetProcAddress(_THIS, const char* proc)
 	return func;
 }
 
-#endif /* HAVE_OPENGL */
+#endif /* SDL_VIDEO_OPENGL */
