@@ -1,42 +1,37 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002  Sam Lantinga
+    Copyright (C) 1997-2006 Sam Lantinga
 
     This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
+    modify it under the terms of the GNU Lesser General Public
     License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
+    version 2.1 of the License, or (at your option) any later version.
 
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
+    Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Library General Public
-    License along with this library; if not, write to the Free
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
     Sam Lantinga
     slouken@libsdl.org
 */
-
-#ifdef SAVE_RCSID
-static char rcsid =
- "@(#) $Id: SDL_dx5events.c,v 1.17 2002/11/09 05:47:24 slouken Exp $";
-#endif
+#include "SDL_config.h"
 
 /* CAUTION!!!!  If you modify this file, check ../windib/SDL_sysevents.c */
 
 #include "directx.h"
 
-#include <stdio.h>
+#include "SDL_main.h"
 #include "SDL_events.h"
 #include "SDL_video.h"
-#include "SDL_error.h"
 #include "SDL_syswm.h"
-#include "SDL_sysevents.h"
-#include "SDL_events_c.h"
-#include "SDL_lowvideo.h"
+#include "../../events/SDL_sysevents.h"
+#include "../../events/SDL_events_c.h"
+#include "../wincommon/SDL_lowvideo.h"
 #include "SDL_dx5video.h"
 
 #ifndef WM_APP
@@ -66,7 +61,25 @@ static SDL_keysym *TranslateKey(UINT scancode, SDL_keysym *keysym, int pressed);
 
 /* DJM: If the user setup the window for us, we want to save his window proc,
    and give him a chance to handle some messages. */
-static WNDPROC userWindowProc = NULL;
+#ifdef STRICT
+#define WNDPROCTYPE	WNDPROC
+#else
+#define WNDPROCTYPE	FARPROC
+#endif
+static WNDPROCTYPE userWindowProc = NULL;
+
+static HWND GetTopLevelParent(HWND hWnd)
+{
+    HWND hParentWnd;
+    while (1)
+    {
+        hParentWnd = GetParent(hWnd);
+        if (hParentWnd == NULL)
+            break;
+        hWnd = hParentWnd;
+    }
+    return hWnd;
+}
 
 /* Convert a DirectInput return code to a text message */
 static void SetDIerror(char *function, int code)
@@ -98,12 +111,13 @@ static void SetDIerror(char *function, int code)
 			error = "Device not initialized";
 			break;
 		default:
-			sprintf(errbuf, "%s: Unknown DirectInput error: 0x%x",
+			SDL_snprintf(errbuf, SDL_arraysize(errbuf),
+			         "%s: Unknown DirectInput error: 0x%x",
 								function, code);
 			break;
 	}
 	if ( ! errbuf[0] ) {
-		sprintf(errbuf, "%s: %s", function, error);
+		SDL_snprintf(errbuf, SDL_arraysize(errbuf), "%s: %s", function, error);
 	}
 	SDL_SetError("%s", errbuf);
 	return;
@@ -141,6 +155,7 @@ static int DX5_DInputInit(_THIS)
 	LPDIRECTINPUTDEVICE device;
 	HRESULT     result;
 	DIPROPDWORD dipdw;
+	HWND        topwnd;
 
 	/* Create the DirectInput object */
 	result = DInputCreate(SDL_Instance, DIRECTINPUT_VERSION,
@@ -167,8 +182,9 @@ static int DX5_DInputInit(_THIS)
 			SetDIerror("DirectInputDevice::QueryInterface", result);
 			return(-1);
 		}
+		topwnd =  GetTopLevelParent(SDL_Window);
 		result = IDirectInputDevice2_SetCooperativeLevel(SDL_DIdev[i],
-					SDL_Window, inputs[i].win_level);
+					topwnd, inputs[i].win_level);
 		if ( result != DI_OK ) {
 			SetDIerror("DirectInputDevice::SetCooperativeLevel",
 									result);
@@ -182,7 +198,7 @@ static int DX5_DInputInit(_THIS)
 		}
 
 		/* Set buffered input -- we aren't polling */
-		memset(&dipdw, 0, sizeof(dipdw));
+		SDL_memset(&dipdw, 0, sizeof(dipdw));
 		dipdw.diph.dwSize = sizeof(dipdw);
 		dipdw.diph.dwHeaderSize = sizeof(dipdw.diph);
 		dipdw.diph.dwObj = 0;
@@ -277,6 +293,11 @@ static void handle_mouse(const int numevents, DIDEVICEOBJECTDATA *ptrbuf)
 	Uint8 button;
 	DWORD timestamp = 0;
 
+	/* Sanity check. Mailing list reports this being NULL unexpectedly. */
+	if (SDL_PublicSurface == NULL) {
+		return;
+	}
+
 	/* If we are in windowed mode, Windows is taking care of the mouse */
 	if (  (SDL_PublicSurface->flags & SDL_OPENGL) ||
 	     !(SDL_PublicSurface->flags & SDL_FULLSCREEN) ) {
@@ -455,8 +476,7 @@ static void handle_mouse(const int numevents, DIDEVICEOBJECTDATA *ptrbuf)
 }
 
 /* The main Win32 event handler */
-LONG
- DX5_HandleMessage(_THIS, HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
+LRESULT DX5_HandleMessage(_THIS, HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
 {
 	switch (msg) {
 #ifdef WM_ACTIVATEAPP
@@ -482,7 +502,7 @@ LONG
 
 #ifdef WM_DISPLAYCHANGE
 		case WM_DISPLAYCHANGE: {
-			WORD BitsPerPixel;
+			WPARAM BitsPerPixel;
 			WORD SizeX, SizeY;
 
 			/* Ack!  The display changed size and/or depth! */
@@ -496,7 +516,10 @@ LONG
 
 		/* The keyboard is handled via DirectInput */
 		case WM_SYSKEYUP:
-		case WM_SYSKEYDOWN:
+		case WM_SYSKEYDOWN: {
+			/* Pass syskey to DefWindwoProc (ALT-F4, etc.) */
+		}
+		break;
 		case WM_KEYUP:
 		case WM_KEYDOWN: {
 			/* Ignore windows keyboard messages */;
@@ -532,13 +555,13 @@ LONG
 				wmmsg.lParam = lParam;
 				posted = SDL_PrivateSysWMEvent(&wmmsg);
 
-         /* DJM: If the user isn't watching for private messages in her
-            SDL event loop, then pass it along to any win32 specific
-            window proc.
-          */
-         } else if (userWindowProc) {
-            return userWindowProc(hwnd, msg, wParam, lParam);
-         }
+			/* DJM: If the user isn't watching for private
+				messages in her SDL event loop, then pass it
+				along to any win32 specific window proc.
+			 */
+			} else if (userWindowProc) {
+				return CallWindowProc(userWindowProc, hwnd, msg, wParam, lParam);
+			}
 		}
 		break;
 	}
@@ -576,7 +599,7 @@ static int DX5_CheckInput(_THIS, int timeout, BOOL processInput)
 			result = IDirectInputDevice2_Poll(SDL_DIdev[i]);
 			if ( (result == DIERR_INPUTLOST) ||
 					(result == DIERR_NOTACQUIRED) ) {
-				if ( strcmp(inputs[i].name, "mouse") == 0 ) {
+				if ( SDL_strcmp(inputs[i].name, "mouse") == 0 ) {
 					mouse_lost = 1;
 				}
 				IDirectInputDevice2_Acquire(SDL_DIdev[i]);
@@ -590,7 +613,7 @@ static int DX5_CheckInput(_THIS, int timeout, BOOL processInput)
 							timeout, QS_ALLEVENTS);
 	if ((event >= WAIT_OBJECT_0) && (event < (WAIT_OBJECT_0+SDL_DIndev))) {
 		DWORD numevents;
-		DIDEVICEOBJECTDATA evtbuf[INPUT_QSIZE];
+		static DIDEVICEOBJECTDATA evtbuf[INPUT_QSIZE];
 
 		event -= WAIT_OBJECT_0;
 		numevents = INPUT_QSIZE;
@@ -599,7 +622,7 @@ static int DX5_CheckInput(_THIS, int timeout, BOOL processInput)
 							evtbuf, &numevents, 0);
 		if ( (result == DIERR_INPUTLOST) ||
 					(result == DIERR_NOTACQUIRED) ) {
-			if ( strcmp(inputs[event].name, "mouse") == 0 ) {
+			if ( SDL_strcmp(inputs[event].name, "mouse") == 0 ) {
 				mouse_lost = 1;
 			}
 			IDirectInputDevice2_Acquire(SDL_DIdev[event]);
@@ -635,6 +658,7 @@ void DX5_DInputReset(_THIS, int fullscreen)
 	DWORD level;
 	int i;
 	HRESULT result;
+	HWND topwnd;
 
 	for ( i=0; i<MAX_INPUTS; ++i ) {
 		if ( SDL_DIdev[i] != NULL ) {
@@ -644,8 +668,9 @@ void DX5_DInputReset(_THIS, int fullscreen)
 				level = inputs[i].win_level;
 			}
 			IDirectInputDevice2_Unacquire(SDL_DIdev[i]);
+			topwnd = GetTopLevelParent(SDL_Window);
 			result = IDirectInputDevice2_SetCooperativeLevel(
-					SDL_DIdev[i], SDL_Window, level);
+					SDL_DIdev[i], topwnd, level);
 			IDirectInputDevice2_Acquire(SDL_DIdev[i]);
 			if ( result != DI_OK ) {
 				SetDIerror(
@@ -678,7 +703,7 @@ void DX5_InitOSKeymap(_THIS)
 	int i;
 
 	/* Map the DIK scancodes to SDL keysyms */
-	for ( i=0; i<SDL_TABLESIZE(DIK_keymap); ++i )
+	for ( i=0; i<SDL_arraysize(DIK_keymap); ++i )
 		DIK_keymap[i] = 0;
 
 	/* Defined DIK_* constants */
@@ -777,7 +802,7 @@ void DX5_InitOSKeymap(_THIS)
 	DIK_keymap[DIK_NUMPADENTER] = SDLK_KP_ENTER;
 	DIK_keymap[DIK_RCONTROL] = SDLK_RCTRL;
 	DIK_keymap[DIK_DIVIDE] = SDLK_KP_DIVIDE;
-	DIK_keymap[DIK_SYSRQ] = SDLK_SYSREQ;
+	DIK_keymap[DIK_SYSRQ] = SDLK_PRINT;
 	DIK_keymap[DIK_RMENU] = SDLK_RALT;
 	DIK_keymap[DIK_PAUSE] = SDLK_PAUSE;
 	DIK_keymap[DIK_HOME] = SDLK_HOME;
@@ -802,11 +827,11 @@ static SDL_keysym *TranslateKey(UINT scancode, SDL_keysym *keysym, int pressed)
 	keysym->sym = DIK_keymap[scancode];
 	keysym->mod = KMOD_NONE;
 	keysym->unicode = 0;
-	if ( pressed && SDL_TranslateUNICODE ) { /* Someday use ToUnicode() */
+	if ( pressed && SDL_TranslateUNICODE ) {
 		UINT vkey;
 #ifndef NO_GETKEYBOARDSTATE
-		BYTE keystate[256];
-		BYTE chars[2];
+		BYTE	keystate[256];
+		Uint16	wchars[2];
 #endif
 
 		vkey = MapVirtualKey(scancode, 1);
@@ -815,16 +840,18 @@ static SDL_keysym *TranslateKey(UINT scancode, SDL_keysym *keysym, int pressed)
 		keysym->unicode = vkey;
 #else
 		GetKeyboardState(keystate);
-		if ( ToAscii(vkey,scancode,keystate,(WORD *)chars,0) == 1 ) {
-			keysym->unicode = chars[0];
+		if (SDL_ToUnicode(vkey, scancode, keystate, wchars, sizeof(wchars)/sizeof(wchars[0]), 0) == 1)
+		{
+			keysym->unicode = wchars[0];
 		}
-#endif
+#endif /* NO_GETKEYBOARDSTATE */
 	}
 	return(keysym);
 }
 
 int DX5_CreateWindow(_THIS)
 {
+	char *windowid = SDL_getenv("SDL_WINDOWID");
 	int i;
 
 	/* Clear out DirectInput variables in case we fail */
@@ -834,24 +861,25 @@ int DX5_CreateWindow(_THIS)
 		SDL_DIfun[i] = NULL;
 	}
 
-#ifndef CS_BYTEALIGNCLIENT
-#define CS_BYTEALIGNCLIENT	0
-#endif
-	SDL_RegisterApp("SDL_app", CS_BYTEALIGNCLIENT, 0);
+	SDL_RegisterApp(NULL, 0, 0);
+
+	SDL_windowid = (windowid != NULL);
 	if ( SDL_windowid ) {
-		SDL_Window = (HWND)strtol(SDL_windowid, NULL, 0);
+		SDL_Window = (HWND)SDL_strtoull(windowid, NULL, 0);
+		if ( SDL_Window == NULL ) {
+			SDL_SetError("Couldn't get user specified window");
+			return(-1);
+		}
 
 		/* DJM: we want all event's for the user specified
-		   window to be handled by SDL.
+			window to be handled by SDL.
 		 */
-		if (SDL_Window) {
-			userWindowProc = (WNDPROC)GetWindowLong(SDL_Window, GWL_WNDPROC);
-			SetWindowLong(SDL_Window, GWL_WNDPROC, (LONG)WinMessage);
-		}
+		userWindowProc = (WNDPROCTYPE)GetWindowLongPtr(SDL_Window, GWLP_WNDPROC);
+		SetWindowLongPtr(SDL_Window, GWLP_WNDPROC, (LONG_PTR)WinMessage);
 	} else {
 		SDL_Window = CreateWindow(SDL_Appname, SDL_Appname,
                         (WS_OVERLAPPED|WS_CAPTION|WS_SYSMENU|WS_MINIMIZEBOX),
-                                 0, 0, 0, 0, NULL, NULL, SDL_Instance, NULL);
+                        CW_USEDEFAULT, CW_USEDEFAULT, 0, 0, NULL, NULL, SDL_Instance, NULL);
 		if ( SDL_Window == NULL ) {
 			SDL_SetError("Couldn't create window");
 			return(-1);
@@ -864,6 +892,12 @@ int DX5_CreateWindow(_THIS)
 		return(-1);
 	}
 
+	/* JC 14 Mar 2006
+		Flush the message loop or this can cause big problems later
+		Especially if the user decides to use dialog boxes or assert()!
+	*/
+	WIN_FlushMessageQueue();
+
 	/* Ready to roll */
 	return(0);
 }
@@ -874,5 +908,16 @@ void DX5_DestroyWindow(_THIS)
 	DX5_DInputQuit(this);
 
 	/* Destroy our window */
-	DestroyWindow(SDL_Window);
+	if ( SDL_windowid ) {
+		SetWindowLongPtr(SDL_Window, GWLP_WNDPROC, (LONG_PTR)userWindowProc);
+	} else {
+		DestroyWindow(SDL_Window);
+	}
+	SDL_UnregisterApp();
+
+	/* JC 14 Mar 2006
+		Flush the message loop or this can cause big problems later
+		Especially if the user decides to use dialog boxes or assert()!
+	*/
+	WIN_FlushMessageQueue();
 }
