@@ -5,6 +5,10 @@
 
 #include "SDL.h"
 
+#ifdef __MACOS__
+#define HAVE_OPENGL
+#endif
+
 #ifdef HAVE_OPENGL
 
 #include "SDL_opengl.h"
@@ -18,8 +22,13 @@
 /* The SDL_OPENGLBLIT interface is deprecated.
    The code is still available for benchmark purposes though.
 */
+
 static SDL_bool USE_DEPRECATED_OPENGLBLIT = SDL_FALSE;
 
+static SDL_Surface *global_image = NULL;
+static GLuint global_texture = 0;
+static GLuint cursor_texture = 0;
+
 /**********************************************************************/
 
 void HotKey_ToggleFullScreen(void)
@@ -118,6 +127,10 @@ void SDL_GL_Enter2DMode()
 	glDisable(GL_CULL_FACE);
 	glEnable(GL_TEXTURE_2D);
 
+	/* This allows alpha blending of 2D textures with the scene */
+	glEnable(GL_BLEND);
+	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+
 	glViewport(0, 0, screen->w, screen->h);
 
 	glMatrixMode(GL_PROJECTION);
@@ -229,21 +242,14 @@ GLuint SDL_GL_LoadTexture(SDL_Surface *surface, GLfloat *texcoord)
 	return texture;
 }
 
-void DrawLogoTexture(void)
+void DrawLogoCursor(void)
 {
-	static GLuint texture;
 	static GLfloat texMinX, texMinY;
 	static GLfloat texMaxX, texMaxY;
-	static int x = 0;
-	static int y = 0;
 	static int w, h;
-	static int delta_x = 1;
-	static int delta_y = 1;
-	static Uint32 last_moved = 0;
+	int x, y;
 
-	SDL_Surface *screen = SDL_GetVideoSurface();
-
-	if ( ! texture ) {
+	if ( ! cursor_texture ) {
 		SDL_Surface *image;
 		GLfloat texcoord[4];
 
@@ -256,7 +262,7 @@ void DrawLogoTexture(void)
 		h = image->h;
 
 		/* Convert the image into an OpenGL texture */
-		texture = SDL_GL_LoadTexture(image, texcoord);
+		cursor_texture = SDL_GL_LoadTexture(image, texcoord);
 
 		/* Make texture coordinates easy to understand */
 		texMinX = texcoord[0];
@@ -268,7 +274,66 @@ void DrawLogoTexture(void)
 		SDL_FreeSurface(image);
 
 		/* Make sure that the texture conversion is okay */
-		if ( ! texture ) {
+		if ( ! cursor_texture ) {
+			return;
+		}
+	}
+
+	/* Move the image around */
+	SDL_GetMouseState(&x, &y);
+	x -= w/2;
+	y -= h/2;
+
+	/* Show the image on the screen */
+	SDL_GL_Enter2DMode();
+	glBindTexture(GL_TEXTURE_2D, cursor_texture);
+	glBegin(GL_TRIANGLE_STRIP);
+	glTexCoord2f(texMinX, texMinY); glVertex2i(x,   y  );
+	glTexCoord2f(texMaxX, texMinY); glVertex2i(x+w, y  );
+	glTexCoord2f(texMinX, texMaxY); glVertex2i(x,   y+h);
+	glTexCoord2f(texMaxX, texMaxY); glVertex2i(x+w, y+h);
+	glEnd();
+	SDL_GL_Leave2DMode();
+}
+
+void DrawLogoTexture(void)
+{
+	static GLfloat texMinX, texMinY;
+	static GLfloat texMaxX, texMaxY;
+	static int x = 0;
+	static int y = 0;
+	static int w, h;
+	static int delta_x = 1;
+	static int delta_y = 1;
+
+	SDL_Surface *screen = SDL_GetVideoSurface();
+
+	if ( ! global_texture ) {
+		SDL_Surface *image;
+		GLfloat texcoord[4];
+
+		/* Load the image (could use SDL_image library here) */
+		image = SDL_LoadBMP(LOGO_FILE);
+		if ( image == NULL ) {
+			return;
+		}
+		w = image->w;
+		h = image->h;
+
+		/* Convert the image into an OpenGL texture */
+		global_texture = SDL_GL_LoadTexture(image, texcoord);
+
+		/* Make texture coordinates easy to understand */
+		texMinX = texcoord[0];
+		texMinY = texcoord[1];
+		texMaxX = texcoord[2];
+		texMaxY = texcoord[3];
+
+		/* We don't need the original image anymore */
+		SDL_FreeSurface(image);
+
+		/* Make sure that the texture conversion is okay */
+		if ( ! global_texture ) {
 			return;
 		}
 	}
@@ -295,7 +360,7 @@ void DrawLogoTexture(void)
 
 	/* Show the image on the screen */
 	SDL_GL_Enter2DMode();
-	glBindTexture(GL_TEXTURE_2D, texture);
+	glBindTexture(GL_TEXTURE_2D, global_texture);
 	glBegin(GL_TRIANGLE_STRIP);
 	glTexCoord2f(texMinX, texMinY); glVertex2i(x,   y  );
 	glTexCoord2f(texMaxX, texMinY); glVertex2i(x+w, y  );
@@ -308,21 +373,16 @@ void DrawLogoTexture(void)
 /* This code is deprecated, but available for speed comparisons */
 void DrawLogoBlit(void)
 {
-	static SDL_Surface *image = NULL;
-	static GLuint texture;
-	static GLfloat texMinX, texMinY;
-	static GLfloat texMaxX, texMaxY;
 	static int x = 0;
 	static int y = 0;
 	static int w, h;
 	static int delta_x = 1;
 	static int delta_y = 1;
-	static Uint32 last_moved = 0;
 
 	SDL_Rect dst;
 	SDL_Surface *screen = SDL_GetVideoSurface();
 
-	if ( image == NULL ) {
+	if ( global_image == NULL ) {
 		SDL_Surface *temp;
 
 		/* Load the image (could use SDL_image library here) */
@@ -334,7 +394,7 @@ void DrawLogoBlit(void)
 		h = temp->h;
 
 		/* Convert the image into the screen format */
-		image = SDL_CreateRGBSurface(
+		global_image = SDL_CreateRGBSurface(
 				SDL_SWSURFACE,
 				w, h,
 				screen->format->BitsPerPixel,
@@ -342,13 +402,13 @@ void DrawLogoBlit(void)
 				screen->format->Gmask,
 				screen->format->Bmask,
 				screen->format->Amask);
-		if ( image ) {
-			SDL_BlitSurface(temp, NULL, image, NULL);
+		if ( global_image ) {
+			SDL_BlitSurface(temp, NULL, global_image, NULL);
 		}
 		SDL_FreeSurface(temp);
 
 		/* Make sure that the texture conversion is okay */
-		if ( ! image ) {
+		if ( ! global_image ) {
 			return;
 		}
 	}
@@ -382,14 +442,14 @@ void DrawLogoBlit(void)
 	dst.y = y;
 	dst.w = w;
 	dst.h = h;
-	SDL_BlitSurface(image, NULL, screen, &dst);
+	SDL_BlitSurface(global_image, NULL, screen, &dst);
 
 	/* Show the image on the screen */
 	SDL_UpdateRects(screen, 1, &dst);
 }
 
 int RunGLTest( int argc, char* argv[],
-               int logo, int slowly, int bpp, float gamma )
+               int logo, int logocursor, int slowly, int bpp, float gamma, int noframe, int fsaa, int sync, int accel )
 {
 	int i;
 	int rgb_size[3];
@@ -438,17 +498,21 @@ int RunGLTest( int argc, char* argv[],
 		video_flags = SDL_OPENGL;
 	}
 	for ( i=1; argv[i]; ++i ) {
-		if ( strcmp(argv[1], "-fullscreen") == 0 ) {
+		if ( strcmp(argv[i], "-fullscreen") == 0 ) {
 			video_flags |= SDL_FULLSCREEN;
 		}
 	}
 
+        if (noframe) {
+           video_flags |= SDL_NOFRAME;
+        }
+
 	/* Initialize the display */
 	switch (bpp) {
 	    case 8:
-		rgb_size[0] = 2;
+		rgb_size[0] = 3;
 		rgb_size[1] = 3;
-		rgb_size[2] = 3;
+		rgb_size[2] = 2;
 		break;
 	    case 15:
 	    case 16:
@@ -467,6 +531,18 @@ int RunGLTest( int argc, char* argv[],
 	SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, rgb_size[2] );
 	SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16 );
 	SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
+	if ( fsaa ) {
+		SDL_GL_SetAttribute( SDL_GL_MULTISAMPLEBUFFERS, 1 );
+		SDL_GL_SetAttribute( SDL_GL_MULTISAMPLESAMPLES, fsaa );
+	}
+	if ( accel ) {
+		SDL_GL_SetAttribute( SDL_GL_ACCELERATED_VISUAL, 1 );
+	}
+	if ( sync ) {
+		SDL_GL_SetAttribute( SDL_GL_SWAP_CONTROL, 1 );
+	} else {
+		SDL_GL_SetAttribute( SDL_GL_SWAP_CONTROL, 0 );
+	}
 	if ( SDL_SetVideoMode( w, h, bpp, video_flags ) == NULL ) {
 		fprintf(stderr, "Couldn't set GL mode: %s\n", SDL_GetError());
 		SDL_Quit();
@@ -491,6 +567,20 @@ int RunGLTest( int argc, char* argv[],
 	printf( "SDL_GL_DEPTH_SIZE: requested %d, got %d\n", bpp, value );
 	SDL_GL_GetAttribute( SDL_GL_DOUBLEBUFFER, &value );
 	printf( "SDL_GL_DOUBLEBUFFER: requested 1, got %d\n", value );
+	if ( fsaa ) {
+		SDL_GL_GetAttribute( SDL_GL_MULTISAMPLEBUFFERS, &value );
+		printf("SDL_GL_MULTISAMPLEBUFFERS: requested 1, got %d\n", value );
+		SDL_GL_GetAttribute( SDL_GL_MULTISAMPLESAMPLES, &value );
+		printf("SDL_GL_MULTISAMPLESAMPLES: requested %d, got %d\n", fsaa, value );
+	}
+	if ( accel ) {
+		SDL_GL_GetAttribute( SDL_GL_ACCELERATED_VISUAL, &value );
+		printf( "SDL_GL_ACCELERATED_VISUAL: requested 1, got %d\n", value );
+	}
+	if ( sync ) {
+		SDL_GL_GetAttribute( SDL_GL_SWAP_CONTROL, &value );
+		printf( "SDL_GL_SWAP_CONTROL: requested 1, got %d\n", value );
+	}
 
 	/* Set the window manager title bar */
 	SDL_WM_SetCaption( "SDL GL test", "testgl" );
@@ -583,7 +673,7 @@ int RunGLTest( int argc, char* argv[],
 			glVertex3fv(cube[2]);
 			glColor3fv(color[7]);
 			glVertex3fv(cube[7]);
-#else // flat cube
+#else /* flat cube */
 			glColor3f(1.0, 0.0, 0.0);
 			glVertex3fv(cube[0]);
 			glVertex3fv(cube[1]);
@@ -634,6 +724,9 @@ int RunGLTest( int argc, char* argv[],
 				DrawLogoTexture();
 			}
 		}
+		if ( logocursor ) {
+			DrawLogoCursor();
+		}
 
 		SDL_GL_SwapBuffers( );
 
@@ -670,6 +763,19 @@ int RunGLTest( int argc, char* argv[],
 			((float)frames/(this_time-start_time))*1000.0);
 	}
 
+	if ( global_image ) {
+	   	SDL_FreeSurface(global_image);
+		global_image = NULL;
+	}
+	if ( global_texture ) {
+		glDeleteTextures( 1, &global_texture );
+		global_texture = 0;
+	}
+	if ( cursor_texture ) {
+		glDeleteTextures( 1, &cursor_texture );
+		cursor_texture = 0;
+	}
+
 	/* Destroy our GL context, etc. */
 	SDL_Quit( );
 	return(0);
@@ -677,11 +783,15 @@ int RunGLTest( int argc, char* argv[],
 
 int main(int argc, char *argv[])
 {
-	int i, logo;
+	int i, logo, logocursor = 0;
 	int numtests;
 	int bpp = 0;
 	int slowly;
 	float gamma = 0.0;
+	int noframe = 0;
+	int fsaa = 0;
+	int accel = 0;
+	int sync = 0;
 
 	logo = 0;
 	slowly = 0;
@@ -698,6 +808,9 @@ int main(int argc, char *argv[])
 			logo = 1;
 			USE_DEPRECATED_OPENGLBLIT = SDL_TRUE;
 		}
+		if ( strcmp(argv[i], "-logocursor") == 0 ) {
+			logocursor = 1;
+		}
 		if ( strcmp(argv[i], "-slow") == 0 ) {
 			slowly = 1;
 		}
@@ -707,15 +820,27 @@ int main(int argc, char *argv[])
 		if ( strcmp(argv[i], "-gamma") == 0 ) {
  		       gamma = (float)atof(argv[++i]);
 		}
+		if ( strcmp(argv[i], "-noframe") == 0 ) {
+ 		       noframe = 1;
+		}
+		if ( strcmp(argv[i], "-fsaa") == 0 ) {
+ 		       ++fsaa;
+		}
+		if ( strcmp(argv[i], "-accel") == 0 ) {
+ 		       ++accel;
+		}
+		if ( strcmp(argv[i], "-sync") == 0 ) {
+ 		       ++sync;
+		}
 		if ( strncmp(argv[i], "-h", 2) == 0 ) {
  		       printf(
-"Usage: %s [-twice] [-logo] [-slow] [-bpp n] [-gamma n]\n",
+"Usage: %s [-twice] [-logo] [-logocursor] [-slow] [-bpp n] [-gamma n] [-noframe] [-fsaa] [-fullscreen]\n",
  			      argv[0]);
 			exit(0);
 		}
 	}
 	for ( i=0; i<numtests; ++i ) {
- 	       RunGLTest(argc, argv, logo, slowly, bpp, gamma);
+ 		RunGLTest(argc, argv, logo, logocursor, slowly, bpp, gamma, noframe, fsaa, sync, accel);
 	}
 	return 0;
 }
